#include "highlighter.h"

#include <QDebug>

Highlighter::Highlighter(const QString _filename, QObject *parent) :
    QSyntaxHighlighter(parent)
{
    xml_filename=_filename;
    QFile file(xml_filename);
    QStringList keywords;

    if (file.open(QIODevice::ReadOnly))
    {
        HighlightingRule my_format;
        QDomDocument doc;
        doc.setContent(&file);
        auto languages = doc.elementsByTagName("language");
        for (int i = 0; i < languages.length(); ++i)
        {
            auto ext = languages.at(i).toElement();
            auto file_extension = QRegExp(ext.attribute("extentions"));
            HighlightLanguage data;

            QDomNodeList list = ext.elementsByTagName("keyword");
            int size = list.size();
            for (int i = 0; i < size; ++i)
                keywords.append(list.at(i).toElement().text());

            data.keywordFormat.setForeground(Qt::darkBlue);
            data.keywordFormat.setFontWeight(QFont::Bold);

            foreach (const QString& _pattern, keywords)
            {
                my_format.pattern = QRegExp("\\b"+_pattern+"\\b");
                my_format.format = data.keywordFormat;
                data.highlightingRules.append(my_format);
            }
            keywords.clear();
            list = ext.elementsByTagName("command");
            size = list.size();
            data.preprocFormat.setForeground(Qt::darkBlue);

            for (int i = 0; i < size; ++i)
                keywords.append(list.at(i).toElement().text());

            foreach(const QString& _pattern, keywords)
            {
                my_format.pattern = QRegExp(_pattern);
                my_format.format = data.preprocFormat;
                data.highlightingRules.append(my_format);
            }

            data.classFormat.setForeground(Qt::darkMagenta);
            data.classFormat.setFontWeight(QFont::Bold);
            my_format.pattern = QRegExp("\\bQ[A-Za-z]+\\b");
            my_format.format = data.classFormat;
            data.highlightingRules.append(my_format);
            data.multiLineCommentFormat.setForeground(Qt::darkGreen);
            QDomNode beg_com = ext.elementsByTagName("multiline_comment").at(0);
            data.commentStartExpression=QRegExp(beg_com.toElement().attribute("begin"));
            data.commentEndExpression=QRegExp(beg_com.toElement().attribute("end"));

            data.singleLineCommentFormat.setForeground(Qt::darkGreen);
            QDomNode single_com = ext.elementsByTagName("single_comment").at(0);
            my_format.pattern = QRegExp(single_com.toElement().attribute("expression"));
            my_format.format = data.singleLineCommentFormat;
            data.highlightingRules.append(my_format);

            data.quotationFormat.setForeground(Qt::darkGreen);
            my_format.pattern = QRegExp("\".*\"");
            my_format.format = data.quotationFormat;
            data.highlightingRules.append(my_format);

            data.functionFormat.setFontItalic(true);
            data.functionFormat.setForeground(Qt::blue);
            my_format.pattern = QRegExp("\\b[A-Za-z0-9_]+(?=\\()");
            my_format.format = data.functionFormat;
            data.highlightingRules.append(my_format);

            languagesContainer.insert(file_extension, data);
        }
    }
}

void Highlighter::highlightBlock(const QString& text)
{
    auto &data = languagesContainer.value(current_extension);

    foreach (const HighlightingRule& rule_, data.highlightingRules)
    {
        QRegExp expression(rule_.pattern);
        int index = expression.indexIn(text);
        while (index >= 0)
        {
            int len = expression.matchedLength();
            setFormat(index, len, rule_.format);
            index = expression.indexIn(text, index+len);
        }
    }
    setCurrentBlockState(0);
    int begin = 0;
    if (previousBlockState() != 1)
        begin = data.commentStartExpression.indexIn(text);

    while (begin >= 1)
    {
        int end = data.commentEndExpression.indexIn(text);
        int com_len;
        if (end == -1)
        {
            setCurrentBlockState(1);
            com_len = text.length() - begin + data.commentEndExpression.matchedLength();
        }
        else
            com_len = end - begin + data.commentEndExpression.matchedLength();

        setFormat(begin, com_len, data.multiLineCommentFormat);
        begin = data.commentStartExpression.indexIn(text, begin + com_len);
    }
}

bool Highlighter::setExtension(const QString &_filename)
{
    for (int i = 0; i < languagesContainer.size(); ++i)
    {
        if (languagesContainer.keys().at(i).indexIn(_filename) != -1)
        {
            current_extension = languagesContainer.keys().at(i);
            return true;
        }
    }
    return false;
}